/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.*;
// JMX
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.*;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class CmpServlet extends EntityServletHelper {

    public void init(ServletConfig config ) throws ServletException {
	super.init(config);
    }

    /** 
     * Creates a VelocityContext. 
     */
    protected Context createContext( HttpServletRequest request, HttpServletResponse response) { 
	HttpSession session = request.getSession();
	// Try to get the initial VelocityContext
	VelocityContext initialContext = (VelocityContext)session.getAttribute("initialContext");
	if (initialContext == null) {
	    // initial context lost, recreate it
	    initialContext = InitVeloContext.doIt(session);
	    logger.log(BasicLevel.DEBUG, "Initial VelocityContext lost. Create, initialize and attach to the session a new initial VelocityContext");
	}
	VelocityContext context = new VelocityContext(initialContext);
	logger.log(BasicLevel.DEBUG, "Create a new VelocityContext containing the initial one");
	return context;
    }

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	String fileName = null;
	String ejbName = null;
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("fileName")) {
		    fileName = request.getParameter("fileName");
		    context.put("fileName", fileName);
		} else if (paramName.equals("ejbName")) {
		    ejbName = request.getParameter("ejbName");
		    context.put("ejbName", ejbName);
		}
	    }
	} catch (ManagementException e) {
	    alertException = Script.alert(e.getMessage());
	    context.put("alertException", alertException);
	}

	// Get the container's name
	String cont_error = "";
	try {
	    String contName = (String)JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), "ContainerName");
	    context.put("contName", contName);
	} catch (Exception e) {
	    cont_error = "Error while getting attribute:" + e.getMessage();
	}
	context.put("cont_error", cont_error);
	
	// Get specific information about the CMP bean
	String info_error = "";
	String cmpName_error = "";
	ObjectName MBeanName = null;
	try {
	    // Get the name of the associated MBean
	    MBeanName = JonasObjectName.ejbcmp(fileName, ejbName);
	    logger.log(BasicLevel.DEBUG, "MBeanName = " + MBeanName.toString());

	    // Get the EJB's class name
	    String ejbClass = (String)JonasManagementRepr.getAttribute(MBeanName, "EJBClass");
	    context.put("ejbClass", ejbClass);

	    getEntityBeanInfo(context, MBeanName, info_error, "passivationTimeOut", "shared", "minPoolSize", "curPoolSize", "maxCacheSize", "curCacheSize", "entityCounters");
	} catch (MalformedObjectNameException me) {
	    cmpName_error = me.getMessage();
	} catch (Exception e) {
	    info_error = e.getMessage();
	}
	context.put("info_error", info_error);
	context.put("cmpName_error", cmpName_error);

	String datasource_error = "";
	getDatasourceInfo(context, MBeanName, datasource_error, "nbDatasources", "dataSources");

	String jmsConnFact_error = "";
	getConnFactoryInfo(context, MBeanName, jmsConnFact_error, "nbConnFactories", "jmsConnFactories");

	String jmsDest_error = "";
	getDestinationInfo(context, MBeanName, jmsDest_error, "nbDestinations", "destinations");

	// Return template
	Template template = null;
	try {
	    template = getTemplate("CMP.vm");
	} catch (ParseErrorException pee) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }
}
